/*
 * rcpi.c - RCPI based STA steering.
 *
 * Copyright (C) 2022 IOPSYS Software Solutions AB. All rights reserved.
 *
 * See LICENSE file for license related information.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include <easy/easy.h>

#include "utils.h"
#include "debug.h"
#include "steer.h"

struct rcpi_steer_control {
	uint8_t diffsnr;
	uint8_t low;
	int8_t hysteresis;
	uint8_t rcpi_threshold;
	void *self;
};

static int rcpi_steer_init(void **priv);
static int rcpi_steer_exit(void *priv);

int rcpi_steer(void *priv, struct steer_sta *s)
{
	struct rcpi_steer_control *p = (struct rcpi_steer_control *)priv;
	struct steer_target *b;

	trace("%s: --------->\n", __func__);
	UNUSED(p);	//TODO

	s->verdict = STEER_VERDICT_UNDECIDED;
	s->reason = STEER_REASON_UNDEFINED;

	if (list_empty(s->bcnlist)) {
		s->verdict = STEER_VERDICT_NOK;
		return 0;
	}

	s->best = list_first_entry(s->bcnlist, struct steer_target, list);
	list_for_each_entry(b, s->bcnlist, list) {
		if ((b->rcpi - s->best->rcpi) > p->diffsnr) {

			/* Use only measurements requested by cntlr explicitly */
			if (!b->requested) {
				dbg("%s: ignoring unexpected bcn metrics\n", __func__);
				continue;
			}

			dbg("%s: new best bcn from "MACFMT" with rcpi %d\n",
			    __func__, MAC2STR(b->bssid), b->rcpi);
			s->best = b;
		}
	}

	/* best bssid is a current one - no need to steer */
	if (!memcmp(s->best->bssid, s->s->bssid, 6)) {
		s->verdict = STEER_VERDICT_NOK;
		return 0;
	}

	s->reason = STEER_REASON_LOW_RCPI;
	s->verdict = STEER_VERDICT_OK;

	return 0;
}

extern struct steer_control rcpi;
struct steer_control rcpi = {
	.name = "rcpi",
	.init = rcpi_steer_init,
	.exit = rcpi_steer_exit,
	.steer = rcpi_steer,
};


static int rcpi_steer_init(void **priv)
{
	struct rcpi_steer_control *p;

	p = calloc(1, sizeof(struct rcpi_steer_control));
	if (!p)
		return -1;

	*priv = p;
	p->self = &rcpi;
	p->low = 90;
	p->rcpi_threshold = 100;
	p->hysteresis = 5;
	p->diffsnr = 10;

	dbg("%s: ========================>\n", __func__);
	return 0;
}

static int rcpi_steer_exit(void *priv)
{
	struct rcpi_steer_control *p = (struct rcpi_steer_control *)priv;

	if (p)
		free(p);

	dbg("%s: <========================\n", __func__);
	return 0;
}

